@@ -1,13 +1,13 @@
 /*********************                                                        */
 /*! \file parser.h
  ** \verbatim
- ** Original author: Morgan Deters
- ** Major contributors: Christopher L. Conway
- ** Minor contributors (to current version): Dejan Jovanovic, Kshitij Bansal, Francois Bobot, Andrew Reynolds
+ ** Top contributors (to current version):
+ **   Morgan Deters, Andrew Reynolds, Christopher L. Conway
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2014  New York University and The University of Iowa
- ** See the file COPYING in the top-level source directory for licensing
- ** information.\endverbatim
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
+ ** in the top-level source directory) and their institutional affiliations.
+ ** All rights reserved.  See the file COPYING in the top-level source
+ ** directory for licensing information.\endverbatim
  **
  ** \brief A collection of state for use by parser implementations.
  **
@@ -30,6 +30,7 @@
 #include <cvc4/expr/symbol_table.h>
 #include <cvc4/expr/kind.h>
 #include <cvc4/expr/expr_stream.h>
+#include <cvc4/util/unsafe_interrupt_exception.h>
 
 namespace CVC4 {
 
@@ -39,12 +40,40 @@ class ExprManager;
 class Command;
 class FunctionType;
 class Type;
+class ResourceManager;
+
+//for sygus gterm two-pass parsing
+class CVC4_PUBLIC SygusGTerm {
+public:
+  enum{
+    gterm_op,
+    gterm_let,
+    gterm_constant,
+    gterm_variable,
+    gterm_input_variable,
+    gterm_local_variable,
+    gterm_nested_sort,
+    gterm_unresolved,
+    gterm_ignore,
+  };
+  Type d_type;
+  Expr d_expr;
+  std::vector< Expr > d_let_vars;
+  unsigned d_gterm_type;
+  std::string d_name;
+  std::vector< SygusGTerm > d_children;
+  
+  unsigned getNumChildren() { return d_children.size(); }
+  void addChild(){
+    d_children.push_back( SygusGTerm() );
+  }
+};
 
 namespace parser {
 
 class Input;
 
-/** Types of check for the symols */
+/** Types of checks for the symbols */
 enum DeclarationCheck {
   /** Enforce that the symbol has been declared */
   CHECK_DECLARED,
@@ -105,9 +134,11 @@ inline std::ostream& operator<<(std::ostream& out, SymbolType type) {
  */
 class CVC4_PUBLIC Parser {
   friend class ParserBuilder;
-
+private:
   /** The expression manager */
   ExprManager *d_exprManager;
+  /** The resource manager associated with this expr manager */
+  ResourceManager *d_resourceManager;
 
   /** The input that we're parsing. */
   Input *d_input;
@@ -133,6 +164,12 @@ class CVC4_PUBLIC Parser {
    */
   size_t d_assertionLevel;
 
+  /**
+   * Whether we're in global declarations mode (all definitions and
+   * declarations are global).
+   */
+  bool d_globalDeclarations;
+
   /**
    * Maintains a list of reserved symbols at the assertion level that might
    * not occur in our symbol table.  This is necessary to e.g. support the
@@ -193,10 +230,14 @@ class CVC4_PUBLIC Parser {
    * "Preemption commands": extra commands implied by subterms that
    * should be issued before the currently-being-parsed command is
    * issued.  Used to support SMT-LIBv2 ":named" attribute on terms.
+   *
+   * Owns the memory of the Commands in the queue.
    */
   std::list<Command*> d_commandQueue;
 
-  /** Lookup a symbol in the given namespace. */
+  /** Lookup a symbol in the given namespace (as specified by the type). 
+   * Only returns a symbol if it is not overloaded, returns null otherwise.
+   */
   Expr getSymbol(const std::string& var_name, SymbolType type);
 
 protected:
@@ -217,9 +258,7 @@ protected:
 
 public:
 
-  virtual ~Parser() {
-    delete d_input;
-  }
+  virtual ~Parser();
 
   /** Get the associated <code>ExprManager</code>. */
   inline ExprManager* getExprManager() const {
@@ -281,21 +320,52 @@ public:
   bool logicIsForced() const { return d_logicIsForced; }
 
   /**
-   * Returns a variable, given a name.
+   * Gets the variable currently bound to name.
    *
    * @param name the name of the variable
    * @return the variable expression
+   * Only returns a variable if its name is not overloaded, returns null otherwise.
    */
   Expr getVariable(const std::string& name);
 
   /**
-   * Returns a function, given a name.
+   * Gets the function currently bound to name.
    *
    * @param name the name of the variable
    * @return the variable expression
+   * Only returns a function if its name is not overloaded, returns null otherwise.
    */
   Expr getFunction(const std::string& name);
 
+  /**
+   * Returns the expression that name should be interpreted as, based on the current binding.
+   *
+   * The symbol name should be declared.
+   * This creates the expression that the string "name" should be interpreted as.
+   * Typically this corresponds to a variable, but it may also correspond to
+   * a nullary constructor or a defined function.
+   * Only returns an expression if its name is not overloaded, returns null otherwise.
+   */
+  virtual Expr getExpressionForName(const std::string& name);
+  
+  /**
+   * Returns the expression that name should be interpreted as, based on the current binding.
+   *
+   * This is the same as above but where the name has been type cast to t. 
+   */
+  virtual Expr getExpressionForNameAndType(const std::string& name, Type t);
+  
+  /**
+   * Returns the kind that should be used for applications of expression fun, where
+   * fun has "function-like" type, i.e. where checkFunctionLike(fun) returns true. 
+   * Returns a parse error if fun does not have function-like type.
+   * 
+   * For example, this function returns
+   *   APPLY_UF if fun has function type, 
+   *   APPLY_CONSTRUCTOR if fun has constructor type.
+   */
+  Kind getKindForFunction(Expr fun);
+  
   /**
    * Returns a sort, given a name.
    * @param sort_name the name to look up
@@ -330,9 +400,10 @@ public:
    * @param notes notes to add to a parse error (if one is generated)
    * @throws ParserException if checks are enabled and the check fails
    */
-  void checkDeclaration(const std::string& name, DeclarationCheck check,
+  void checkDeclaration(const std::string& name,
+                        DeclarationCheck check,
                         SymbolType type = SYM_VARIABLE,
-                        std::string notes = "") throw(ParserException);
+                        std::string notes = "");
 
   /**
    * Reserve a symbol at the assertion level.
@@ -340,12 +411,15 @@ public:
   void reserveSymbolAtAssertionLevel(const std::string& name);
 
   /**
-   * Checks whether the given name is bound to a function.
-   * @param name the name to check
-   * @throws ParserException if checks are enabled and name is not
-   * bound to a function
+   * Checks whether the given expression is function-like, i.e.
+   * it expects arguments. This is checked by looking at the type 
+   * of fun. Examples of function types are function, constructor,
+   * selector, tester.
+   * @param fun the expression to check
+   * @throws ParserException if checks are enabled and fun is not
+   * a function
    */
-  void checkFunctionLike(const std::string& name) throw(ParserException);
+  void checkFunctionLike(Expr fun);
 
   /**
    * Check that <code>kind</code> can accept <code>numArgs</code> arguments.
@@ -355,7 +429,7 @@ public:
    * <code>kind</code> cannot be applied to <code>numArgs</code>
    * arguments.
    */
-  void checkArity(Kind kind, unsigned numArgs) throw(ParserException);
+  void checkArity(Kind kind, unsigned numArgs);
 
   /**
    * Check that <code>kind</code> is a legal operator in the current
@@ -366,54 +440,84 @@ public:
    * @throws ParserException if the parser mode is strict and the
    * operator <code>kind</code> has not been enabled
    */
-  void checkOperator(Kind kind, unsigned numArgs) throw(ParserException);
+  void checkOperator(Kind kind, unsigned numArgs);
 
-  /**
-   * Returns the type for the variable with the given name.
+  /** Create a new CVC4 variable expression of the given type. 
    *
-   * @param var_name the symbol to lookup
-   * @param type the (namespace) type of the symbol
+   * flags specify information about the variable, e.g. whether it is global or defined
+   *   (see enum in expr_manager_template.h).
+   *
+   * If a symbol with name already exists,
+   *  then if doOverload is true, we create overloaded operators.
+   *  else if doOverload is false, the existing expression is shadowed by the new expression.
    */
-  Type getType(const std::string& var_name, SymbolType type = SYM_VARIABLE);
-
-  /** Create a new CVC4 variable expression of the given type. */
   Expr mkVar(const std::string& name, const Type& type,
-             uint32_t flags = ExprManager::VAR_FLAG_NONE);
+             uint32_t flags = ExprManager::VAR_FLAG_NONE, 
+             bool doOverload = false);
 
   /**
    * Create a set of new CVC4 variable expressions of the given type.
+   *
+   * flags specify information about the variable, e.g. whether it is global or defined
+   *   (see enum in expr_manager_template.h).
+   *
+   * For each name, if a symbol with name already exists,
+   *  then if doOverload is true, we create overloaded operators.
+   *  else if doOverload is false, the existing expression is shadowed by the new expression.
    */
   std::vector<Expr>
     mkVars(const std::vector<std::string> names, const Type& type,
-           uint32_t flags = ExprManager::VAR_FLAG_NONE);
+           uint32_t flags = ExprManager::VAR_FLAG_NONE, 
+           bool doOverload = false);
 
   /** Create a new CVC4 bound variable expression of the given type. */
   Expr mkBoundVar(const std::string& name, const Type& type);
 
   /**
    * Create a set of new CVC4 bound variable expressions of the given type.
+   *
+   * flags specify information about the variable, e.g. whether it is global or defined
+   *   (see enum in expr_manager_template.h).
+   *
+   * For each name, if a symbol with name already exists,
+   *  then if doOverload is true, we create overloaded operators.
+   *  else if doOverload is false, the existing expression is shadowed by the new expression.
    */
   std::vector<Expr> mkBoundVars(const std::vector<std::string> names, const Type& type);
 
   /** Create a new CVC4 function expression of the given type. */
   Expr mkFunction(const std::string& name, const Type& type,
-                  uint32_t flags = ExprManager::VAR_FLAG_NONE);
+                  uint32_t flags = ExprManager::VAR_FLAG_NONE, 
+                  bool doOverload=false);
 
   /**
    * Create a new CVC4 function expression of the given type,
    * appending a unique index to its name.  (That's the ONLY
    * difference between mkAnonymousFunction() and mkFunction()).
+   *
+   * flags specify information about the variable, e.g. whether it is global or defined
+   *   (see enum in expr_manager_template.h).
    */
   Expr mkAnonymousFunction(const std::string& prefix, const Type& type,
                            uint32_t flags = ExprManager::VAR_FLAG_NONE);
 
-  /** Create a new variable definition (e.g., from a let binding). */
+  /** Create a new variable definition (e.g., from a let binding). 
+   * levelZero is set if the binding must be done at level 0.
+   * If a symbol with name already exists,
+   *  then if doOverload is true, we create overloaded operators.
+   *  else if doOverload is false, the existing expression is shadowed by the new expression.
+   */
   void defineVar(const std::string& name, const Expr& val,
-                 bool levelZero = false);
+                 bool levelZero = false, bool doOverload = false);
 
-  /** Create a new function definition (e.g., from a define-fun). */
+  /** Create a new function definition (e.g., from a define-fun). 
+   * levelZero is set if the binding must be done at level 0.
+   * If a symbol with name already exists,
+   *  then if doOverload is true, we create overloaded operators.
+   *  else if doOverload is false, the existing expression is shadowed by the new expression.
+   */
   void defineFunction(const std::string& name, const Expr& val,
-                      bool levelZero = false);
+                      bool levelZero = false, bool doOverload = false);
 
   /** Create a new type definition. */
   void defineType(const std::string& name, const Type& type);
@@ -463,9 +567,77 @@ public:
 
   /**
    * Create sorts of mutually-recursive datatypes.
+   * For each symbol defined by the datatype, if a symbol with name already exists,
+   *  then if doOverload is true, we create overloaded operators.
+   *  else if doOverload is false, the existing expression is shadowed by the new expression.
    */
   std::vector<DatatypeType>
-  mkMutualDatatypeTypes(const std::vector<Datatype>& datatypes);
+  mkMutualDatatypeTypes(std::vector<Datatype>& datatypes, bool doOverload=false);
+
+  /** make flat function type
+   *
+   * Returns the "flat" function type corresponding to the function taking
+   * argument types "sorts" and range type "range".  A flat function type is
+   * one whose range is not a function. Notice that if sorts is empty and range
+   * is not a function, then this function returns range itself.
+   *
+   * If range is a function type, we add its function argument sorts to sorts
+   * and consider its function range as the new range. For each sort S added
+   * to sorts in this process, we add a new bound variable of sort S to
+   * flattenVars.
+   *
+   * For example:
+   * mkFlattenFunctionType( { Int, (-> Real Real) }, (-> Int Bool), {} ):
+   * - returns the the function type (-> Int (-> Real Real) Int Bool)
+   * - updates sorts to { Int, (-> Real Real), Int },
+   * - updates flattenVars to { x }, where x is bound variable of type Int.
+   *
+   * Notice that this method performs only one level of flattening, for example,
+   * mkFlattenFunctionType({ Int, (-> Real Real) }, (-> Int (-> Int Bool)), {}):
+   * - returns the the function type (-> Int (-> Real Real) Int (-> Int Bool))
+   * - updates sorts to { Int, (-> Real Real), Int },
+   * - updates flattenVars to { x }, where x is bound variable of type Int.
+   *
+   * This method is required so that we do not return functions
+   * that have function return type (these give an unhandled exception
+   * in the ExprManager). For examples of the equivalence between function
+   * definitions in the proposed higher-order extension of the smt2 language,
+   * see page 3 of http://matryoshka.gforge.inria.fr/pubs/PxTP2017.pdf.
+   *
+   * The argument flattenVars is needed in the case of defined functions
+   * with function return type. These have implicit arguments, for instance:
+   *    (define-fun Q ((x Int)) (-> Int Int) (lambda y (P x)))
+   * is equivalent to the command:
+   *    (define-fun Q ((x Int) (z Int)) Int (@ (lambda y (P x)) z))
+   * where @ is (higher-order) application. In this example, z is added to
+   * flattenVars.
+   */
+  Type mkFlatFunctionType(std::vector<Type>& sorts,
+                          Type range,
+                          std::vector<Expr>& flattenVars);
+
+  /** make flat function type
+   *
+   * Same as above, but does not take argument flattenVars.
+   * This is used when the arguments of the function are not important (for
+   * instance, if we are only using this type in a declare-fun).
+   */
+  Type mkFlatFunctionType(std::vector<Type>& sorts, Type range);
+
+  /** make higher-order apply
+   *
+   * This returns the left-associative curried application of (function) expr to
+   * the arguments in args, starting at index startIndex.
+   *
+   * For example, mkHoApply( f, { a, b }, 0 ) returns
+   *  (HO_APPLY (HO_APPLY f a) b)
+   *
+   * If args is non-empty, the expected type of expr is (-> T0 ... Tn T), where
+   *    args[i-startIndex].getType() = Ti
+   * for each i where startIndex <= i < args.size(). If expr is not of this
+   * type, the expression returned by this method will not be well typed.
+   */
+  Expr mkHoApply(Expr expr, std::vector<Expr>& args, unsigned startIndex = 0);
 
   /**
    * Add an operator to the current legal set.
@@ -485,8 +657,10 @@ public:
   /** Is the symbol bound to a boolean variable? */
   bool isBoolean(const std::string& name);
 
-  /** Is the symbol bound to a function (or function-like thing)? */
-  bool isFunctionLike(const std::string& name);
+  /** Is fun a function (or function-like thing)? 
+  * Currently this means its type is either a function, constructor, tester, or selector.
+  */
+  bool isFunctionLike(Expr fun);
 
   /** Is the symbol bound to a defined function? */
   bool isDefinedFunction(const std::string& name);
@@ -498,26 +672,21 @@ public:
   bool isPredicate(const std::string& name);
 
   /** Parse and return the next command. */
-  Command* nextCommand() throw(ParserException);
+  Command* nextCommand();
 
   /** Parse and return the next expression. */
-  Expr nextExpression() throw(ParserException);
+  Expr nextExpression();
 
   /** Issue a warning to the user. */
-  inline void warning(const std::string& msg) {
-    d_input->warning(msg);
-  }
-
+  void warning(const std::string& msg) { d_input->warning(msg); }
   /** Issue a warning to the user, but only once per attribute. */
   void attributeNotSupported(const std::string& attr);
 
   /** Raise a parse error with the given message. */
-  inline void parseError(const std::string& msg) throw(ParserException) {
-    d_input->parseError(msg);
-  }
-
+  inline void parseError(const std::string& msg) { d_input->parseError(msg); }
   /** Unexpectedly encountered an EOF */
-  inline void unexpectedEOF(const std::string& msg) throw(ParserException) {
+  inline void unexpectedEOF(const std::string& msg)
+  {
     d_input->parseError(msg, true);
   }
 
@@ -535,7 +704,8 @@ public:
    * support parsing quantifiers (just not doing anything with them).
    * So this mechanism gives you a way to do it with --parse-only.
    */
-  inline void unimplementedFeature(const std::string& msg) throw(ParserException) {
+  inline void unimplementedFeature(const std::string& msg)
+  {
     if(!d_parseOnly) {
       parseError("Unimplemented feature: " + msg);
     }
@@ -561,6 +731,14 @@ public:
     }
   }
 
+  virtual void reset() {
+    d_symtab->reset();
+  }
+
+  void setGlobalDeclarations(bool flag) {
+    d_globalDeclarations = flag;
+  }
+
   /**
    * Set the current symbol table used by this parser.
    * From now on, this parser will perform its definitions and
@@ -616,8 +794,32 @@ public:
   public:
     ExprStream(Parser* parser) : d_parser(parser) {}
     ~ExprStream() { delete d_parser; }
-    Expr nextExpr() { return d_parser->nextExpression(); }
+    Expr nextExpr() override { return d_parser->nextExpression(); }
   };/* class Parser::ExprStream */
+  
+  //------------------------ operator overloading
+  /** is this function overloaded? */
+  bool isOverloadedFunction(Expr fun) {
+    return d_symtab->isOverloadedFunction(fun);
+  }
+  
+  /** Get overloaded constant for type.
+   * If possible, it returns a defined symbol with name
+   * that has type t. Otherwise returns null expression.
+  */
+  Expr getOverloadedConstantForType(const std::string& name, Type t) {
+    return d_symtab->getOverloadedConstantForType(name, t);
+  }
+  
+  /**
+   * If possible, returns a defined function for a name
+   * and a vector of expected argument types. Otherwise returns
+   * null expression.
+   */
+  Expr getOverloadedFunctionForTypes(const std::string& name, std::vector< Type >& argTypes) {
+    return d_symtab->getOverloadedFunctionForTypes(name, argTypes);
+  }
+  //------------------------ end operator overloading
 };/* class Parser */
 
 }/* CVC4::parser namespace */
